/* Jim Susinno - eyefractfuncs.c
 * by David Eyerman
 *
 *  from glfract v0.0.6
 *
 */

//#define M_PI 3.1415927

/* iterate_func(...): from David Eyerman's Eyefract v0.5.6
 *
 *
 *
 */

#ifdef USE_C_PLUS_PLUS

int iterate_func(int mi, complex *z, double a, double b){
    /*  iterate function, return number of steps until bail  */
    static complex t;
#else
int iterate_func(int mi, struct complex *z, double a, double b){
    /*  iterate function, return number of steps until bail  */
    static struct complex t;
#endif


    int n=0; //, m=0;

    t.a = z->a;
    t.b = z->b;

    //    switch (which_func){
    switch ( curState->func_idx ) {

        case 0:
        default:
            while ((((t.a*t.a)+(t.b*t.b))<4) && (n<mi)){
                /*  f(z) = z^2 + c  */
                z->a = (t.a * t.a) - (t.b * t.b) + a;
                z->b = (2 * t.a * t.b) + b;

                t.a = z->a;
                t.b = z->b;

                n++;
            }
            break;

       case 1:
            while ((((t.a*t.a)+(t.b*t.b))<4) && (n<mi)){
                /*  f(z) = z^4 + c  */
                z->a = (t.a * t.a * t.a * t.a) - (6 * t.a * t.a * t.b * t.b) + (t.b * t.b * t.b * t.b) + a;
                z->b = 4 * t.a * t.b * (t.a * t.a - t.b * t.b) + b;

                t.a = z->a;
                t.b = z->b;

                n++;
            }
            break;

        case 2:
            while ((((t.a*t.a)+(t.b*t.b))<4) && (n<mi)){
                /*  f(z) = z^4 - z + c  */
                z->a = (t.a * t.a * t.a * t.a) - (6 * t.a * t.a * t.b * t.b) + (t.b * t.b * t.b * t.b) - t.a + a;
                z->b = 4 * t.a * t.b * (t.a * t.a - t.b * t.b) - t.b + b;

                t.a = z->a;
                t.b = z->b;

                n++;
            }
            break;

        case 3:
            while ((((t.a*t.a)+(t.b*t.b))<4) && (n<mi)){
                /*  f(z) = z^5 + c  */
                z->a = (t.a * t.a * t.a * t.a * t.a) - (10 * t.a * t.a * t.a * t.b * t.b) + (5 * t.a * t.b * t.b * t.b * t.b) + a;
                z->b = (t.b * t.b * t.b * t.b * t.b) - (10 * t.a * t.a * t.b * t.b * t.b) + (5 * t.a * t.a * t.a * t.a * t.b) + b;

                t.a = z->a;
                t.b = z->b;

                n++;
            }
            break;

        case 4:
            while ((((t.a*t.a)+(t.b*t.b))<4) && (n<mi)){
                /*  f(z) = z^2 + cz  */
                z->a = (t.a * a) + (t.a * t.a) - (t.b * b) - (t.b * t.b);
                z->b = (t.a * b) + (t.b * a) + (2 * t.a * t.b);

                t.a = z->a;
                t.b = z->b;

                n++;
            }
            break;

        case 5:
            /*  f(z) = pi * i * e^z  */
            while ((t.a<10) && (n<mi)) {
                z->a = -1 * M_PI * exp(t.a) * cos(t.b);
                z->b = M_PI * exp(t.a) * sin(t.b);

                t.a = z->a;
                t.b = z->b;

                n++;
            }
            break;

        case 6:
            /*  f(z) = sin z  */
            while ((t.a<50) && (n<mi)) {
                z->a = sin(t.a)*(exp(t.b)+exp(-1*t.b))/2;
                z->b = cos(t.a)*(exp(t.b)-exp(-1*t.b))/2;

                t.a = z->a;
                t.b = z->b;

                n++;
            }
            break;

        case 7:
            /*  f(z) = a * cos z  */
            while ((t.a<50) && (n<mi)) {
                z->a = (cos(t.a)*(exp(t.b)+exp(-1*t.b))*a - sin(t.a)*(exp(-1*t.b)-exp(t.b))*b)/2;
                z->b = (sin(t.a)*(exp(-1*t.b)-exp(t.b))*a + cos(t.a)*(exp(-1*t.b)-exp(t.b))*b)/2;

                t.a = z->a;
                t.b = z->b;

                n++;
            }
            break;

    }

    return n;

    /* pi * i * tan z */

    /* (1 + .1*i)sin z */

    /* e^(z-1) */

    /* (1/e + .1) * e^z */

    /* (.6 + .8i) * sin z */
}






/* create_fractal(...): from David Eyerman's Eyefract v0.5.6
 *
 *
 *
 */
int create_fractal(double xmin, double xmax, double ymin, double ymax, int abortable){
    /*  the "meat" of the program  */


#ifdef USE_C_PLUS_PLUS
	static complex z;
#else
    static struct complex z;
#endif


    int tempx, tempy, w, h, mi; //, d, 
    int ty, n; //, pw, ph; //, cc=0;
    //    static char t[32];
    
     static double xfa, yfa; //, ww, hh;
     //    int aborted=0;


    w  = curState->width;
    h  = curState->height;
    mi = curState->MAXITS;
//    d  = curState->depth;
    ty = curState->type;
//    pw = curState->pwidth;
//    ph = curState->pheight;


    /* for every pixel on the screen, translate the pixel coordinates
     * to Cartesian coordinates on the scale of [xmin, xmax] and [ymin, ymax].
     * then iterate the function for that  z=(x+yi)  and record n, the number
     * of iterations before we either got over 2 or reached the maximum
     * number of iterations, MAXITS  */

    /*  ratio of coordinate width/height to pixel width/height  */
    xfa = (xmax-xmin)/w;
    yfa = (ymax-ymin)/h;

    /*  ratio of preview pixmap width/height to actual pixmap width/height  *
    ww = (double) pw / w;
    hh = (double) ph / h;
*/
 
    most_hits = 0;

    for (tempy=0; tempy<h; tempy++){

    for (tempx=0; tempx<w; tempx++){

            if (ty) {
                curState->subA.a = xmin + (tempx * xfa);
                curState->subA.b = ymin +  (tempy * yfa);
                z.a = z.b = 0;
            }
            else {
                z.a = xmin + (tempx * xfa);
                z.b = ymin + (tempy * yfa);
            }



            /*
             * assign values to fractal landscape
             *
             *   buffers HERE
             *
             */
            
            n = iterate_func(mi, &z, curState->subA.a, curState->subA.b);
	    if ( n > most_hits ) { most_hits = n; }
            fractal_landscapes[tempx][tempy][current_cache] = n;

	    if ( use_colormap ) {
	      fractal_image[tempx + tempy*h][0] = (GLubyte)(colormap[0][n]/255);
	      fractal_image[tempx + tempy*h][1] = (GLubyte)(colormap[1][n]/255);
	      fractal_image[tempx + tempy*h][2] = (GLubyte)(colormap[2][n]/255);
	    }
	    else {
	      fractal_image[tempx + tempy*h][0] = 124;
	      fractal_image[tempx + tempy*h][1] = 255-n;
	      fractal_image[tempx + tempy*h][2] = n;
	    }

        } // x-for
    } //y-for


  return 0;
}

/****** 
 * END Eyefract code
 ******/


