/* Jim Susinno - glf_display.c
 *   #included from glfract.c
 *
 *  from glfract v0.0.9
 *
 *  ___Globals:___
 *  zoom_params[][]
 *  current_zoom_params[][]
 *
 */




/* DISPLAY FUNCTION
 *  (main display loop)(
 *    
 */
void display() { 
  int ii, jj, kk; 
  char buf[120];
 
  GLenum draw_enum;


  /* Update zoom params cache:
   * 
   *  derive xmin, xmax, ymin, ymax from
   *   center(x,y) and delta_x, delta)y
   *
   */
  if ( mouse_flts[5][0] > 1 ) {
    zoom_params[0][current_cache] = mouse_flts[4][0] - mouse_flts[5][0]/2.0;
    zoom_params[1][current_cache] = mouse_flts[4][0] + mouse_flts[5][0]/2.0;
    zoom_params[2][current_cache] = mouse_flts[4][1] - mouse_flts[5][0]/2.0;	
    zoom_params[3][current_cache] = mouse_flts[4][1] + mouse_flts[5][0]/2.0; 
  }
  else {
    zoom_params[0][current_cache] = mouse_flts[4][0] - (1.0/(2-mouse_flts[5][0]))/2.0;
    zoom_params[1][current_cache] = mouse_flts[4][0] + (1.0/(2-mouse_flts[5][0]))/2.0;
    zoom_params[2][current_cache] = mouse_flts[4][1] - (1.0/(2-mouse_flts[5][0]))/2.0;
    zoom_params[3][current_cache] = mouse_flts[4][1] + (1.0/(2-mouse_flts[5][0]))/2.0;
  }


  curState->subA.a = mouse_flts[6][0];
  curState->subA.b = mouse_flts[6][1];


  /*** create fractal in landscape map ***/
  if ( live_update ) {
    current_zoom_params[0] = zoom_params[0][current_cache];
    current_zoom_params[1] = zoom_params[1][current_cache];
    current_zoom_params[2] = zoom_params[2][current_cache];
    current_zoom_params[3] = zoom_params[3][current_cache];    

    create_fractal( zoom_params[0][current_cache], 
		    zoom_params[1][current_cache], 
		    zoom_params[2][current_cache], 
		    zoom_params[3][current_cache], 1 );
  }


  /*** Begin OpenGL code ***/
  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );


  /* block - 3D fractal view 
   *
   *
   */
  if ( threed_view_on ) {

    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();
    gluLookAt( mouse_flts[2][0], -mouse_flts[1][1], mouse_flts[2][1],
	       0.0, 0.0, 0.0,
	       0.0, 1.0, 0.0 );


    /*** colored origin ***/
    if ( origin_on ) {
      glPushMatrix();
        glScalef( 10.0, 10.0, 10.0 );
	glLineWidth( 3.0 );
	
	glBegin( GL_LINES );
          glColor3f( 1.0, 0.0, 0.0 );  glVertex3f( 0.0, 0.0, 0.0 );
	  glVertex3f( 1.0, 0.0, 0.0 );
	 glEnd();
	 glBegin( GL_LINES );
	  glColor3f( 0.0, 1.0, 0.0 );  glVertex3f( 0.0, 0.0, 0.0 );
	  glVertex3f( 0.0, 1.0, 0.0 );
         glEnd();
         glBegin( GL_LINES );
	  glColor3f( 0.0, 0.0, 1.0 );  glVertex3f( 0.0, 0.0, 0.0 );
	  glVertex3f( 0.0, 0.0, 1.0 );
        glEnd();
	
      glPopMatrix();
    }


    if ( yflip ) { glScalef( 1.0, 1.0, -1.0 ); }


    /* main fractal drawing loop:
     *   iterate through fractal landscape array:{
     *      set color to current point's num_hits
     *      set height to same
     *      draw a line from the current point to the next point
     *   }
     *
     */
    //for ( kk=0; kk<cache_level; kk++ ) {
    for ( kk=0; kk<1; kk++ ) {

      /*** set up state ***/
      glPushMatrix();

        /*** fractal spatial display range scale ***/
        glScalef( mouse_flts[3][1], draw_height,  mouse_flts[3][1] );

	glColor3f( 0.5, 0.5, 0.5 );
	glLineWidth( 1.0 );
	
	/*** keep fractal centered in world ***/
	glTranslatef( -0.5, 0.0, -0.5 );
	
	/*** global floor ***/
	if ( floor_on ) {
	  glColor3f( 0.0, 1.0, 0.8 );
	  draw_floor( 0.0, 1.0, 0.0, 1.0 );
	}

	/* draw range indicator when not updating map every loop -
	 *  scale mouse_flts params to [0,1] scale of current_zoom_params.
	 *  The box will show the user where a zoom-in will
	 *  take them on the fractal.
	 */
	if ( !live_update ) {
	  float dim[4];
	  
	  glPushMatrix();

	    glTranslatef( 0.0, draw_height, 0.0 );

	    /*** draw zoom indicator ***/
	    glColor3f( 1.0, 1.0, 1.0 );
	    glLineWidth( 2.0 );
	    
	    dim[0] = (zoom_params[0][current_cache] - current_zoom_params[0])
	      / (current_zoom_params[1] - current_zoom_params[0]);
	    dim[1] = (zoom_params[1][current_cache] - current_zoom_params[0])
	      / (current_zoom_params[1] - current_zoom_params[0]);
  
	    dim[2] = (zoom_params[2][current_cache] - current_zoom_params[2])
	      / (current_zoom_params[3] - current_zoom_params[2]);
	    dim[3] = (zoom_params[3][current_cache] - current_zoom_params[2])
	      / (current_zoom_params[3] - current_zoom_params[2]);
	    
	    draw_floor( dim[0], dim[1], dim[2], dim[3] ); 
	  
	  glPopMatrix();
	}


	/* 2D fractal draw loop: ii->X, jj->Z
	 *
	 *  vertices range X:[0,1] Y:[0,1] Z:[0,1]
	 *
	 *
	 */
	glLineWidth( 1.0 );

	switch( draw_mode ) {
 	 case 0: draw_enum = GL_LINE_STRIP; break;
	 case 1: draw_enum = GL_POLYGON; break;
	 case 2: draw_enum = GL_LINE_LOOP; break;
	 case 3: draw_enum = GL_LINES; break;		  
	 default: draw_enum = GL_LINE_STRIP; break;
	}

	for ( ii=0; ii<curState->width - 1; ii++ ) {
	  for ( jj=0; jj<curState->height - 1; jj++ ) {    
	    int xflags[4] = { 1, 0, 0, 1 };
	    int yflags[4] = { 0, 0, 1, 1 };
	    int np = 3;
	    int qq=0;

	    if ( draw_mode == 1 ) { np = 4; }

	    glBegin( draw_enum );

	      for ( qq=0; qq<np; qq++ ) {
		int xf = xflags[qq];
		int yf = yflags[qq];
		
		if ( use_colormap ) {
		  glColor3f( (float)colormap[0][ fractal_landscapes[ii+xf][jj+yf][kk] ]/65536.0,
			     (float)colormap[1][ fractal_landscapes[ii+xf][jj+yf][kk] ]/65536.0,
			     (float)colormap[2][ fractal_landscapes[ii+xf][jj+yf][kk] ]/65536.0  );
		}
		else {
		  glColor3f( 0.6,
			     1.0 - (float)fractal_landscapes[ii+xf][jj+yf][kk]/(float)most_hits,
			     (float)fractal_landscapes[ii+xf][jj+yf][kk]/(float)most_hits );
		}
		
		glVertex3f( (float)(ii+xf)/(float)curState->width, 
			    fractal_landscapes[ii+xf][jj+yf][kk],
			    (float)(jj+yf)/(float)curState->height );
		
	      }
	      
	    glEnd();  


	  } /*** landscape loop ii ***/
	}  /*** landscape loop jj ***/
	
      glPopMatrix();

    }  /*** kk loop ***/

  }  /*** done drawing 3D fractal ***/
  


  /* Set up and display info overlay
   *
   *
   */
  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();
  glMatrixMode( GL_PROJECTION );
  glPushMatrix();
    glLoadIdentity();
    glOrtho( 0, current_width, 0, current_height, -1.0, 1.0 );
    glDisable( GL_LIGHTING );
    glDisable( GL_DEPTH_TEST ); 
    glColor4f( 0.6, 1.0, 0.6, 1.0 );
    

    /* raster fractal - preview view
     *
     *
     */
    if ( twod_view_on ) {
      /*** the border ***/
      glLineWidth( 1.0 );
      glBegin( GL_LINE_LOOP );
        glVertex2f( 9, 100 );
	glVertex2f( 10 + curState->width, 100 );
	glVertex2f( 10 + curState->width, 100 + curState->height  );
	glVertex2f( 9, 100 + curState->height );
      glEnd();

      /*** the image ***/
      glRasterPos2i( 10, 100 );
      glDrawPixels( curState->width, curState->height, 
		    GL_RGB, GL_UNSIGNED_BYTE, 
		    fractal_image );
      glFlush();
    }


    /* shell display code
     *
     */
    if ( shell_active == 1 ) {
      int i;

      /*** draw a black square under cmd line for contrast ***/
      glColor4f( 0.0, 0.0, 0.0, 1.0 );
      glBegin( GL_POLYGON );
        glVertex2f( 10, 10 );
	glVertex2f( 10, 90 );
	glVertex2f( current_width - 10, 90 );
	glVertex2f( current_width - 10, 10 );
      glEnd();
      
      /*** for mouse variables at left ***/ 
      glBegin( GL_POLYGON );
        glVertex2f( 10, current_height - 10 );
	glVertex2f( 10, current_height - 100 );
	glVertex2f( 260, current_height - 100 );
	glVertex2f( 260, current_height - 10 );
      glEnd();
      
      /*** for state variables at left ***/ 
      glBegin( GL_POLYGON );
        glVertex2f( 10, current_height - 140 );
	glVertex2f( 10, current_height - 220 );
	glVertex2f( 220, current_height - 220 );
	glVertex2f( 220, current_height - 140 );
      glEnd();
   
      /*** for state variables at right ***/  
      glBegin( GL_POLYGON );
        glVertex2f( current_width - 10, current_height - 10 );
	glVertex2f( current_width - 10, current_height - 180 );
	glVertex2f( current_width - 155, current_height - 180 );
	glVertex2f( current_width - 155, current_height - 10 );
      glEnd();
   

      glColor4f( 0.6, 1.0, 0.6, 1.0 );
      
      /*** delimiting box ***/
      glLineWidth( 1.0 ); 
      glBegin( GL_LINE_LOOP );
        glVertex2f( 10, 10 );
        glVertex2f( 10, 90 );
	glVertex2f( current_width - 10, 90 );
	glVertex2f( current_width - 10, 10 );
      glEnd();

     
      /* COMMAND BOX INFORMATION:
       * - command string
       * - mouse mode(s)
       * - state parameters
       */
      sprintf( buf,"Mouse: %s, %s, %s -- ^q(fractal) ^w(view)  ^x==iterate", 
	       mouse_strs[ mouseb[0] ],
	       mouse_strs[ mouseb[1] ],
	       mouse_strs[ mouseb[2] ] ); 
      glRasterPos2i( 20, 65 );
      screenPrintString( GLUT_BITMAP_HELVETICA_12, buf );
      

      /*** command buffer ***/
      glRasterPos2i( 20, 25 );
      screenPrintString( GLUT_BITMAP_HELVETICA_12, cmd );

      /*** display all mouse value states 1-6 ***/
      for ( i=1; i<7; i++ ) {
	 sprintf( buf, "%d] %s:  %f   %f", 
		  i, 
		  mouse_strs[i], 
		  mouse_flts[i][0], 
		  mouse_flts[i][1] );
	 glRasterPos2i( 20, current_height - 10 - 14*i );
	 screenPrintString( GLUT_BITMAP_HELVETICA_12, buf );
      }

      /*** mouse function indicators ***/
      for ( i=0; i<3; i++ ) {

	if ( i==0 ) { glColor3f( 1.0, 0.0, 0.0 ); }
	if ( i==1 ) { glColor3f( 0.0, 1.0, 0.0 ); }
	if ( i==2 ) { glColor3f( 0.0, 0.0, 1.0 ); }

	glBegin( GL_LINES );
	  glVertex2f( 5, current_height - 6 - 14*mouseb[i] );
	  glVertex2f( 15, current_height - 6 - 14*mouseb[i] );
	glEnd();
      }

      glColor4f( 0.6, 1.0, 0.6, 1.0 );

      /* Fractal display dimension information
       *   xmin, xmax, ymin, ymax...
       *
       */
      sprintf( buf, "xmin: %f", zoom_params[0][current_cache] );
      glRasterPos2i( current_width - 100, current_height - 25 );
      screenPrintString( GLUT_BITMAP_HELVETICA_12, buf );

      sprintf( buf, "x_center: %f", (
	zoom_params[0][current_cache]+zoom_params[1][current_cache])/2.0 );
      glRasterPos2i( current_width - 150, current_height - 40 );
      screenPrintString( GLUT_BITMAP_HELVETICA_12, buf );


      sprintf( buf, "x_delta: %f", (
	zoom_params[1][current_cache]-zoom_params[0][current_cache]) );
      glRasterPos2i( current_width - 150, current_height - 55 );
      screenPrintString( GLUT_BITMAP_HELVETICA_12, buf );


      sprintf( buf, "xmax: %f", zoom_params[1][current_cache] );
      glRasterPos2i( current_width - 100, current_height - 70 );
      screenPrintString( GLUT_BITMAP_HELVETICA_12, buf );



      sprintf( buf, "ymin: %f", zoom_params[2][current_cache] );
      glRasterPos2i( current_width - 100, current_height - 115 );
      screenPrintString( GLUT_BITMAP_HELVETICA_12, buf );
      


      sprintf( buf, "y_center: %f", (
	zoom_params[2][current_cache]+zoom_params[3][current_cache])/2.0 );
      glRasterPos2i( current_width - 150, current_height - 130 );
      screenPrintString( GLUT_BITMAP_HELVETICA_12, buf );

      sprintf( buf, "y_delta: %f", (
	zoom_params[3][current_cache]-zoom_params[2][current_cache]) );
      glRasterPos2i( current_width - 150, current_height - 145 );
      screenPrintString( GLUT_BITMAP_HELVETICA_12, buf );


      sprintf( buf, "ymax: %f", zoom_params[3][current_cache] );
      glRasterPos2i( current_width - 100, current_height - 160 );
      screenPrintString( GLUT_BITMAP_HELVETICA_12, buf );


		
      /*** current state information ***/
      sprintf( buf, "resolution: [%d x %d]", curState->width, curState->height );
      glRasterPos2i( 20, current_height - 150 );
      screenPrintString( GLUT_BITMAP_HELVETICA_12, buf );


      sprintf( buf, "maxits, depth, type: [%d %d  %d: %d]", 
	       curState->MAXITS, curState->depth, curState->type, curState->func_idx );
      glRasterPos2i( 20, current_height - 165 );
      screenPrintString( GLUT_BITMAP_HELVETICA_12, buf );

      sprintf( buf, "a, b: [%f %f]", 
	       curState->subA.a, curState->subA.b );
      glRasterPos2i( 20, current_height - 180 );
      screenPrintString( GLUT_BITMAP_HELVETICA_12, buf );


      sprintf( buf, "Live: %d      Y flip: %d", live_update, yflip );
      glRasterPos2i( 20, current_height - 195 );
      screenPrintString( GLUT_BITMAP_HELVETICA_12, buf );

      sprintf( buf, "draw height: %f", draw_height );
      glRasterPos2i( 20, current_height - 210 );
      screenPrintString( GLUT_BITMAP_HELVETICA_12, buf );

    } 



    /* crosshair + line indicator from last click - draw
     *    while dragging, and only if turned on.
     *
     */
    if ( last_drag && mouse_button_down ) {
      int cross_length = 10;

      glColor4f( 0.6, 1.0, 0.6, 1.0 );
      glLineWidth( 1.0 );
      glBegin( GL_LINES );
       glVertex2f( last_click_x + cross_length, current_height - last_click_y );
       glVertex2f( last_click_x - cross_length, current_height - last_click_y );
      glEnd();
      glBegin( GL_LINES );
       glVertex2f( last_click_x, current_height - last_click_y + cross_length );
       glVertex2f( last_click_x, current_height - last_click_y - cross_length );
      glEnd();
      glBegin( GL_LINES );
        glVertex2f( last_click_x, current_height - last_click_y );
        glVertex2f( newx, current_height - newy );
      glEnd();
    }



  glPopMatrix();


  

  glutSwapBuffers();
}

