/* Jim Susinno - glf_keyboard.c
 *   #included from glfract.c
 *
 * from glfract v0.0.9: ORIGIN command, FLAT, SMOOTH, FLOOR
 *
 */



/* Keyboard function
 *   handles key press events
 *
 */
void keyboard( unsigned char key, int x, int y ) {
  int i;

  /*** for command input/checking ***/
  char *delimiters = "! \t\n";
  char *arguments[20];
  char *tok, *cpy;
 

  /*** Escape key - toggle shell active ***/
  if ( key == 27 ) {
    shell_active = 1 - shell_active;
    glutPostRedisplay();
    return;
  }


  if ( shell_active == 1 ) {
    
    /* switch statement for shell-like command processing
     *
     *
     */
    switch ( key ) {
  
  
      /* Control-a: move cursor to beginning of buffer
       *   ctrl-a maps to (char)1
       *
       */
      case 1:
	for ( i=cidx; i>=0; i-- ) { cmd[i] = cmd[i-1]; }
	cmd[0] = cursor;
	cidx = 0;
	break;
    
      
      /* Control-s: move cursor to end of buffer
       *   ctrl-s maps to (char)19
       *
       */
      case 19:
	for ( i=cidx; i<clast; i++ ) { cmd[i] = cmd[i+1]; }
	cmd[clast] = cursor;
	cidx = clast;
	break;
    

 
      /* Control-z: clear cmd buffer 
       *   ctrl-z maps to (char)26
       *
       */
      case 26:
	cmd[0] = cursor;
	for ( i=1; i<clast; i++ ) { cmd[i] = '\0'; }
	cidx = 0;
	clast = 0;
	break;


      
      /* Control-x: 
       *   ctrl-x maps to (char)24
       *
       */
      case 24:
	
	current_zoom_params[0] = zoom_params[0][current_cache];
	current_zoom_params[1] = zoom_params[1][current_cache];
	current_zoom_params[2] = zoom_params[2][current_cache];
	current_zoom_params[3] = zoom_params[3][current_cache];
	
	create_fractal( zoom_params[0][current_cache], 
			zoom_params[1][current_cache], 
			zoom_params[2][current_cache], 
			zoom_params[3][current_cache], 1 );
	/*
	  current_cache++;
	  current_cache %= cache_level;
	  create_fractal( xmin, xmax, ymin, ymax, 1 );
	*/
	
	break;

      
      
      /* control-q : (char)17
       *
       */
      case 17:   
	mouseb[0] = 4;
	mouseb[1] = 5;
	mouseb[2] = 6; 
	break;


      /* control-w : (char)23
       *
       */
      case 23: 
	mouseb[0] = 1;
	mouseb[1] = 2;
	mouseb[2] = 3;
	break;


      /*** 4: EOF, quit ***/
      case 4: exit( 0 ); break;
	

      /* Return - do command {
       *  - first delete the cursor from the command buffer;
       *  - tokenize the command into arguments array(keep count)
       *  - check for '!' to pass to shell
       *  - check command vocabulary, if found, execute;
       *  - if not found, insert the command into the list;
       *  - delete the old cmd string;
       *  }
       */
      case 13:

	/*** delete cursor from buffer ***/
	for ( i=cidx; i<=clast; i++ ) { cmd[i] = cmd[i+1]; }
	
	/*** tokenize string into args array ***/
	i = 0;
	cpy = strdup( cmd );
	tok = strtok( cpy, delimiters );
	while ( tok != NULL ) {
	  arguments[i++] = strdup( tok );
	  tok = strtok( 0, delimiters );
	}
	
	/*** capitalize first word of non-shell lines ***/
	if ( cmd[0] != '!' ) {
	  for ( i=0; i<clast; i++ ) {
	    
	    if ( cmd[i] == 32 ) { i = clast; } 
	    
	    /*** lower case ***/
	    if ( ( cmd[i] >= 97 ) && ( cmd[i] <= 122 ) ) {
	      cmd[i] -= 32;
	    }
	  }
	}
	
	/* Modeler Command vocabulary here - not case
	 *   sensitive thanks to the windows API.
	 *
	 *  
	 *  help
	 *  ?
	 *  quit
	 *  exit
	 */
	
	/*** v0.2.1 - new mouse handler code ***/
	if ( !strncmp( cmd, "MOUS", 4 ) ) { 
	  
	  if ( arguments[1] == NULL ) {
	    printf( "MOUSE HELP:\n" );
	    /*** print mouse function names ***/
	    for ( i=0; i<6; i++ ) {
	      printf( "%d]: %s\n", i, mouse_strs[i] );
	    }     
	  }
	  
	  /*** check here for l,m,r(and caps) ***/
	  else if ( ( arguments[1][0] == 'l' ) ||
		    ( arguments[1][0] == 'L' ) ) {
	    i = 0;
	  }
	  else if ( ( arguments[1][0] == 'm' ) ||
		    ( arguments[1][0] == 'M' ) ) {
	    i = 1;
	  }
	  else if ( ( arguments[1][0] == 'r' ) ||
		    ( arguments[1][0] == 'R' ) ) {
	    i = 2;
	  }
	  else { i = atoi( arguments[1] ) - 1; }
	  
	  /*** bounds check ***/
	  if ( ( i >= 0 ) && ( i <= 2 ) ) {
	    mouseb[i] = atoi( arguments[2] );
	  }      
	  if ( mouseb[i] < 0 ) { mouseb[i] = 0; } 
	  if ( mouseb[i] > 15 ) { mouseb[i] = 0; }
	}
	
	
	/* load command - glfract v0.0.7
	 *
	 *
	 */
	else if ( !strncmp( cmd, "LOAD", 4 ) ) { 
	  if ( i >= 2 ) {
	    colorcount = 0;
	    printf( "loading %s.\n", arguments[1] );
	    ReadFile( arguments[1] ); 
	    /*
	      create_fractal( zoom_params[0][current_cache], 
	      zoom_params[1][current_cache], 
	      zoom_params[2][current_cache], 
	      zoom_params[3][current_cache], 1 );
	    */
	  }
	}
	

	/* save command - glfract v0.1.0
	 *
	 *
	 */
	else if ( !strncmp( cmd, "SAVE", 4 ) ) { 
	  if ( i >= 2 ) {
	    //    colorcount = 0;
	    printf( "saving %s.\n", arguments[1] );
	    WriteFile( arguments[1] ); 	
	  }
	}


	else if ( !strncmp( cmd, "RESET", 5 ) ) { 

	  printf( "Full state reset\n" );
	  
	  //      floor_on = 1;
	  last_drag = 1;
	  mouse_button_down = 0;
	  yflip = 0;
	  draw_height = 0.3;
	  draw_mode = 0;
	  curState->width = curState->height = 50;
	  live_update = 0;
	  
	  twod_view_on = threed_view_on = 1;
	  origin_on = 1;
	  
	  
	  mouseb[0] = 1;
	  mouseb[1] = 2;
	  mouseb[2] = 3;
	  for ( i=0; i<16; i++ ) {
	    mouse_flts[i][0] = mouse_defaults[i][0];
	    mouse_flts[i][1] = mouse_defaults[i][1];
	  }
	  
	}
	
	
	
	else if ( !strncmp( cmd, "YFLIP", 5 ) ) {      
	  if ( arguments[1] != NULL ) {
	    yflip = atoi( arguments[1] );
	  }
	}
	else if ( !strncmp( cmd, "HEIGHT", 6 ) ) { 
	  if ( arguments[1] != NULL ) {
	    draw_height = atof( arguments[1] );
	  }
	}
	else if ( !strncmp( cmd, "DRAWMODE", 8 ) ) { 
	  if ( arguments[1] != NULL ) {
	    draw_mode = atoi( arguments[1] );
	  }
	}
	
	else if ( !strncmp( cmd, "XRES", 4 ) ) { 
	  if ( arguments[1] != NULL ) {
	    curState->width = atoi( arguments[1] );
	  }
	  /*** error check ***/
	  if ( ( curState->width < 5 ) || ( curState->width > LANDSCAPE_SIZE ) ) {
	    curState->width = 100;
	  }
	}
	
	else if ( !strncmp( cmd, "YRES", 4 ) ) { 
	  if ( arguments[1] != NULL ) {
	    curState->height = atoi( arguments[1] );
	  }
	  /*** error check ***/
	  if ( ( curState->height < 5 ) || ( curState->height > LANDSCAPE_SIZE ) ) {
	    curState->height = 100;
	  }
	}
	
	else if ( !strncmp( cmd, "RES", 3 ) ) { 
	  if ( arguments[1] != NULL ) {
	    curState->height = curState->width = atoi( arguments[1] );
	  }
	  /*** error check ***/
	  if ( ( curState->height < 5 ) || ( curState->height > LANDSCAPE_SIZE ) ) {
	    curState->height = curState->width = 100;
	  }
	}

	else if ( !strncmp( cmd, "LIVE", 4 ) ) {      
	  if ( arguments[1] != NULL ) {
	    live_update = atoi( arguments[1] );
	  }
	}
	
	
	/*** v0.0.8 ***/
	else if ( !strncmp( cmd, "2D", 2 ) ) {
	  if ( arguments[1] != NULL ) {
	    twod_view_on = atoi( arguments[1] );
	  }
	}
	else if ( !strncmp( cmd, "3D", 2 ) ) {
	  if ( arguments[1] != NULL ) {
	    threed_view_on = atoi( arguments[1] );
	  }
	}
	else if ( !strncmp( cmd, "ORIGIN", 6 ) ) {
	  if ( arguments[1] != NULL ) {
	    origin_on = atoi( arguments[1] );
	  }
	}
	else if ( !strncmp( cmd, "COLORMAP", 8 ) ) {
	  if ( arguments[1] != NULL ) {
	    use_colormap = atoi( arguments[1] );
	  }
	} 
	else if ( !strncmp( cmd, "DRAG", 4 ) ) {
	  if ( arguments[1] != NULL ) {
	    last_drag = atoi( arguments[1] );
	  }
	} 
	else if ( !strncmp( cmd, "FLOOR", 5 ) ) {
	  if ( arguments[1] != NULL ) {
	    floor_on = atoi( arguments[1] );
	  }
	} 
	
	/* func command - glfract v0.1.0
	 *  changes iteration function to one of
	 *   7 implemented funcs in Eyefractv0.5.6-1 
	 *
	 */
	else if ( !strncmp( cmd, "FUNC", 4 ) ) { 
	  if ( arguments[1] != NULL ) {
	    curState->func_idx = atoi( arguments[1] );
	    /*** bound check ***/
	    if ( ( curState->func_idx < 0 ) || ( curState->func_idx > 7 ) ) {
	      curState->func_idx = 0;
	    }
	  }
	}



	else if ( !strncmp( cmd, "FLAT", 4 ) ) {
	  glShadeModel(GL_FLAT); 
	}
	else if ( !strncmp( cmd, "SMOOTH", 6 ) ) {
	  glShadeModel(GL_SMOOTH); 
	}
	
	else if ( !strncmp( cmd, "HELP", 4 ) ) { 
	  printhelp();
	}
	else if ( !strncmp( cmd, "?", 1 ) ) { 
	  printhelp();
	}
	else if ( !strncmp( cmd, "QUIT", 4 ) ) { 
	  exit( 0 );
	}
	else if ( !strncmp( cmd, "EXIT", 4 ) ) { 
	  exit( 0 );
	}
	
	/*** doesn't match any command ***/
	else { 
	}

	/*** delete old cmd string ***/
	cmd[0] = cursor;
	for ( i=1; i<clast; i++ ) { cmd[i] = '\0'; }
	cidx = clast = 0;
	break;






      /* Backspace - remove char from cmd 
       *
       */
      case 8:
	/*** cmd buffer has data ***/
	if ( clast > 0 ) { 
	  /*** not at beginning ***/
	  if ( cidx > 0 ) {
	    /*** shift back rest of buffer ***/
	    for ( i=cidx; i<=clast+1; i++ ) { 
	      cmd[i-1] = cmd[i]; 
	    }
	    /*** update cursor, end pointer  ***/
	    cidx--;
	    cmd[cidx] = cursor;
	    cmd[clast] = '\0';
	    clast--;
	  }
	}
	break;


      /* Other keys - insert into cmd buffer at cursor point 
       *
       */
      default:
	clast++;
	/*** shift all after cursor forward ***/
	for ( i=clast; i>cidx; i-- ) {
	  cmd[i+1] = cmd[i];
	}
	/*** add char, put cursor after it ***/
	cmd[cidx] = (char)key;
	cidx++;
	cmd[cidx] = cursor;
	break;
    }
    
    
    
  }
  
  else {
    
    switch (key) {
    
    case '-':
      view_angle += 5;
      glMatrixMode( GL_PROJECTION );
      glLoadIdentity();
       gluPerspective( view_angle, 
          (GLfloat) current_width/(GLfloat)current_height, 1.0, 2000.0 );
       glMatrixMode( GL_MODELVIEW );
       glLoadIdentity();
       glTranslatef( 0.0, 0.0, -5.0 );
       glutPostRedisplay();
    break;

    case '=':
       view_angle -= 5;
       glMatrixMode( GL_PROJECTION );
       glLoadIdentity();
       gluPerspective( view_angle, 
	     (GLfloat) current_width/(GLfloat)current_height, 1.0, 2000.0 );
       glMatrixMode( GL_MODELVIEW );
       glLoadIdentity();
       glTranslatef( 0.0, 0.0, -5.0 );
       glutPostRedisplay();
    break;

    case 'h': case 'H': case '?':
    printhelp();
    break;

    
      /* Control-x: 
       *   ctrl-x maps to (char)24
       *
       */
    case 24:

      current_zoom_params[0] = zoom_params[0][current_cache];
      current_zoom_params[1] = zoom_params[1][current_cache];
      current_zoom_params[2] = zoom_params[2][current_cache];
      current_zoom_params[3] = zoom_params[3][current_cache];


      create_fractal( zoom_params[0][current_cache], 
		      zoom_params[1][current_cache], 
		      zoom_params[2][current_cache], 
		      zoom_params[3][current_cache], 1 );
      /*
      current_cache++;
      current_cache %= cache_level;
      create_fractal( xmin, xmax, ymin, ymax, 1 );
      */

      break;

      
      
      /* control-q : (char)17
       *
       */
    case 17:   
      mouseb[0] = 4;
      mouseb[1] = 5;
      mouseb[2] = 6; 
      break;


      /* control-w : (char)23
       *
       */
    case 23: 
      mouseb[0] = 1;
      mouseb[1] = 2;
      mouseb[2] = 3;
      break;


     /*** 4: EOF, quit ***/
    case 4: exit( 0 ); break;

    default: 
     printf( "%d: char %c\n", key, key );
     break;
  }

  }

  /*** update map after every command ***
  current_zoom_params[0] = zoom_params[0][current_cache];
  current_zoom_params[1] = zoom_params[1][current_cache];
  current_zoom_params[2] = zoom_params[2][current_cache];
  current_zoom_params[3] = zoom_params[3][current_cache];


  create_fractal( zoom_params[0][current_cache], 
		  zoom_params[1][current_cache], 
		  zoom_params[2][current_cache], 
		  zoom_params[3][current_cache], 1 );
*/
  
  glutPostRedisplay();
}



