/* glfract.cpp - Jim Susinno 2001
 *
 * Fractal Viewer in 3D
 * 
 * v0.1.0: save command
 *
 *
 */

#define USE_C_PLUS_PLUS



#include <stdio.h>
#include <GL/glut.h>
#include <math.h>

#ifndef NO_STRDUP
 #include <string.h>    /*** string functions ***/
 #include <stdlib.h>    /*** atof, atoi, malloc ***/
#else
 #include "strdup.h"    /*** includes string and stdlib ***/
#endif



#include "complex.h"
#include "state.h"




#define LANDSCAPE_SIZE 1000
#define NUM_CACHE 4
#define MAX_COLORS 3000


char *version = "GLFract v0.1.0";


/*** viewing window parameters ***/
static double view_angle = 85.0;
static int current_width = 150, current_height = 150;
GLfloat hither = 0.02, yon = 3000.0;

/*** mouse motion ***/
int oldx, oldy, newx, newy;
int which_button = -1;

/*** mouse click/drag crosshair/line ***/
int last_click_x, last_click_y;
int shell_active = 1;
int last_drag = 1;
int mouse_button_down = 0;

/*** incoming command from keyboard ***/
char cmd[500];
int cidx = 0;
int clast = 0;
int linenum = 0;
char cursor = '|';

/* from mosh v0.2.4:
 *   multiple changeable
 *    mouse parameters and defaults
 *
 */
GLfloat mouse_flts[16][2];
GLfloat mouse_defaults[16][2] = {
  { 0.0, 0.0 },  /*** 0: --- ***/
  { 0.0, -50.0 },  /*** 1: observer height ***/
  { 13.0, 52.0 },  /*** 2: observer x/z (glide) ***/
  { 0.0, 100.0 },  /*** 3: draw scale ***/
  { 0.0, 0.0 },  /*** 4: center(x,y) ***/
  { 3.4, 0.0 },  /*** 5: dx, dy(width/height) ***/
  { -0.3456, -0.5768 },  /*** 6: fractal scale ***/
  { 0.0, 0.0 },  /*** 7: --- ***/
  { 0.0, 0.0 },
  { 0.0, 0.0 },
  { 0.0, 0.0 },
  { 0.0, 0.0 },
  { 0.0, 0.0 },
  { 0.0, 0.0 },
  { 0.0, 0.0 },
  { 0.0, 0.0 }
};

char *mouse_strs[16] = {
  "----",  
  "height",  
  "glide",  
  "draw scale",  
  "center(x,y)",  
  "dx/dy(width/height)",  
  "Julia param(a,b)",  
  "-----",  
  "--unused--", "--unused--",  "--unused--",  "--unused--",   
  "--unused--", "--unused--",  "--unused--",  "--unused--",   
};

/*** initialize mouse actions ***/
int mouseb[3] = { 1, 2, 3 };



/* the last few fractal states and 
 *  images - NUM_CACHE back into the past.
 */
int fractal_landscapes[LANDSCAPE_SIZE][LANDSCAPE_SIZE][NUM_CACHE];

float zoom_params[5][NUM_CACHE];
float current_zoom_params[5];
float relative_zoom_params[5];

int colormap[3][MAX_COLORS];
int colorcount = 0;

int most_hits = 0;  /*** for color scaling ***/

int current_cache = 0;
int cache_level = 2;

GLfloat global_threshold = 1.0;
int which_func = 0;



/*** glfract version 0.0.5 ***/
int yflip = 0;
float draw_height = 0.1;
int draw_mode = 0;
int live_update = 0;

/*** v0.0.8 ***/
/*** for overview image ***/
GLubyte fractal_image[LANDSCAPE_SIZE * LANDSCAPE_SIZE][3];

int threed_view_on = 1;
int twod_view_on = 0;
int origin_on = 1;
int use_colormap = 0;
int floor_on = 0;

/*** code split up for readability ***/
#ifdef USE_C_PLUS_PLUS
  state *curState;
#else
  #include "eyefractcode.h"
  struct state *curState;
#endif




#include "eyefractfuncs.c"


int ReadLineFromFile( FILE *fp );
int ReadFirstLineFromFile( FILE *fp );
void ReadFile( char *sin );

/*** version 0.1.0 ***/
void WriteFile( char *sin );







/* prints a string to OpenGL viewport
 *
 */
static void screenPrintString( void *font, char *str ) {
  int i, l = strlen(str);
  for( i=0; i<l; i++ ) {
    glutBitmapCharacter(font,*str++);
  }
}


/* draws a grid of specified size
 *
 *
 */
void draw_floor( double xlow, double xhigh, double ylow, double yhigh ) {
  double i,j;
  double xstep;
  double ystep;
  double temp;

  if ( xlow == xhigh ) { return; }
  if ( xlow > xhigh ) { 
     temp = xhigh;
     xhigh = xlow;
     xlow = temp;
  }

  if ( ylow == yhigh ) { return; }
  if ( ylow > yhigh ) { 
     temp = yhigh;
     yhigh = ylow;
     ylow = temp;
  }

  xstep = ( xhigh - xlow ) / 5.0;
  ystep = ( yhigh - ylow ) / 5.0;

  for( i=xlow; i<=xhigh; i+= xstep ) {  
    glBegin( GL_LINES );
      glVertex3f( (float)i, 0.0, ylow );
      glVertex3f( (float)i, 0.0, yhigh );
    glEnd();
  }
  
  for( j=ylow; j<=yhigh; j+= ystep ) {
    glBegin( GL_LINES );
      glVertex3f( xlow, 0.0, (float)j );
      glVertex3f( xhigh, 0.0, (float)j );
    glEnd();
  }
}


/*** Prints help message to stdout ***/
void printhelp() {
  printf( "\nFractal help:\n" );
  printf( "\n");
  printf( "Use the mouse buttons to navigate the world.\n");
  printf( "Type commands the the prompt.  Try mouse.\n");

  printf( "__Command list:__\n" );
  printf( "load, mouse, reset, yflip, height, drawmode, \n" );
  printf( "res, live, 2d, 3d, help, ?, quit, exit.\n" );
  
}


/*** display, mouse and keyboard functions for glut ***/
#include "glf_display.c"
#include "glf_keyboard.c"
#include "glf_mouse.c"




/* Reshape function
 *   handles window resize events
 *
 */
void reshape( int w, int h ) {
  glViewport( 0, 0, (GLsizei)w, (GLsizei)h );
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  gluPerspective( view_angle, (GLfloat) w/(GLfloat)h, hither, yon );
  glMatrixMode( GL_MODELVIEW );
  current_width = w;
  current_height = h;
}





/* reads the first line of a  {*.msn, *.jsn} file 
 *   from file stream.
 *
 * adapted from David Eyerman's Eyefractv0.5.6-1
 *
 * reads: 
 *  xmin, xmax, ymin, ymax, a, b, maxits, xres, yres, func
 *
 *
 */
int ReadFirstLineFromFile( FILE *fp ) {
  char line_in[1500];
  char *tok, *incopy;
  char *delims = "[]:, \t\n";

  /***  grab line from file ***/
  if ( fgets( line_in, 1500, fp ) == NULL ) { return -1; }

  /*** skip blanks and comments ***/
  if ( line_in[0] == '\n' ) { return 0; }
  if ( line_in[0] == '#' ) { return 0; }
 
  if ( strlen(line_in) == 0 ) { return 0; }

  /*** make copy of input and tokenize on delimiters ***/
  incopy = strdup( line_in );

  /*** xmin, xmax, ymin, ymax (numbers 0-3 in array notation) ***/
  tok = strtok( incopy, delims );
  current_zoom_params[0] = atof( tok );

  tok = strtok( 0, delims );
  current_zoom_params[1] = atof( tok );

  tok = strtok( 0, delims );
  current_zoom_params[2] = atof( tok );

  tok = strtok( 0, delims );
  current_zoom_params[3] = atof( tok );

  /*** assign variables to mouse registers ***/
  mouse_flts[4][0] = ( current_zoom_params[0] + current_zoom_params[1] ) / 2.0;
  mouse_flts[4][1] = ( current_zoom_params[2] + current_zoom_params[3] ) / 2.0;
  mouse_flts[5][0] = current_zoom_params[1] - current_zoom_params[0];


  /*** complex params (#s 4,5) ***/
  tok = strtok( 0, delims );
  mouse_flts[6][0] = curState->subA.a = atof( tok );

  tok = strtok( 0, delims );
  mouse_flts[6][1] = curState->subA.b = atof( tok );

  /*** maxits (#6) ***/
  tok = strtok( 0, delims );
  curState->MAXITS = atoi( tok );

  /*** xres, yres(7,8) ***/
  tok = strtok( 0, delims );
  tok = strtok( 0, delims );
 
  /*** which funtion(9) ***/
  tok = strtok( 0, delims );
  curState->func_idx = atoi( tok );

  free( incopy );
  return 0;
}



/* reads a line of a {*.msn, *.jsn} file from file stream.
 *
 * adapted from David Eyerman's Eyefractv0.5.6-1
 *
 * reads a line of colormap info - r,g,b triplet
 *  in 16-bit ints [0-65535].
 */
int ReadLineFromFile( FILE *fp ) {
  char line_in[1500];
  char *tok, *incopy;
  char *delims = "[]:, \t\n";

  /***  grab line from file ***/
  if ( fgets( line_in, 1500, fp ) == NULL ) { return -1; }

  /*** skip blanks and comments ***/
  if ( line_in[0] == '\n' ) { return 0; }
  if ( line_in[0] == '#' ) { return 0; }
 
  if ( strlen(line_in) == 0 ) { return 0; }

  /*** make copy of input and tokenize on delimiters ***/
  incopy = strdup( line_in );
  tok = strtok( incopy, delims );

  colormap[0][colorcount] = atoi( tok );
  
  /*** get next number ***/
  tok = strtok( 0, delims );
  colormap[1][colorcount] = atoi( tok );

  tok = strtok( 0, delims );
  colormap[2][colorcount] = atoi( tok );

  colorcount++;

  free( incopy );
  return 0;
}




/* file reader
 *
 *
 */
void ReadFile( char *sin ) {
  int i;
  char *filename;
  FILE *fp;

  if ( sin != NULL ) {
    filename = strdup( sin );
  }
  else { filename = "numbers.jsn"; }

  if ( ( fp = fopen( filename, "r" ) ) != NULL ) {
    i = ReadFirstLineFromFile( fp );
    if ( i < 0 ) {
      printf( "error reading from first line in file %s.\n", filename );
    }
    else {
      while ( ReadLineFromFile( fp ) > -1 ) {}
    }
    fclose( fp );
  }

  twod_view_on = 1;
  use_colormap = 1;
}




/* file writer
 *
 *
 */
void WriteFile( char *sin ) {
  int i;
  char *filename;
  FILE *fp;

  if ( sin != NULL ) {
    filename = strdup( sin );
  }
  else { filename = "numbers.jsn"; }

  /*** open in write mode; fresh file ***/
  if ( ( fp = fopen( filename, "w" ) ) != NULL ) {

    /*** write all data ***/
    fprintf( fp, "%f %f %f %f %f %f %d %d %d %d\n",
	     current_zoom_params[0], 
	     current_zoom_params[1], 
	     current_zoom_params[2], 
	     current_zoom_params[3], 

	     /*	     curState->xmin,  curState->xmax,
		     curState->ymin,  curState->ymax,*/
	     curState->subA.a, curState->subA.b,
	     curState->MAXITS,
	     curState->width, curState->height, 
	     curState->func_idx );

    /*** write colormap ***/
    for ( i=0; i<colorcount; i++ ) {
      fprintf( fp, "%d %d %d\n", 
	       colormap[0][i],  
	       colormap[1][i],  
	       colormap[2][i] );
    }

    fclose( fp );
  }
}





/*** init() - called from main before glutMainLoop() ***/
void init() {
  int i, j;


  glShadeModel(GL_FLAT); 

  /*** for small sub-image ***/
  glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
 
  /*** set mouse motion vars to defaults ***/
  for ( i=0; i<16; i++ ) {
    mouse_flts[i][0] = mouse_defaults[i][0];
    mouse_flts[i][1] = mouse_defaults[i][1];
  }

  /*** sets zoom parameters to defaults ***/
  for ( j=0; j<NUM_CACHE; j++ ) {
    zoom_params[0][j] = -1.0;
    zoom_params[1][j] = 1.0;
    zoom_params[2][j] = -1.0;
    zoom_params[3][j] = 1.0;
  }

  /*
  current_zoom_params[0] = zoom_params[0][current_cache];
  current_zoom_params[1] = zoom_params[1][current_cache];
  current_zoom_params[2] = zoom_params[2][current_cache];
  current_zoom_params[3] = zoom_params[3][current_cache];
  
  create_fractal( zoom_params[0][current_cache], 
		  zoom_params[1][current_cache], 
		  zoom_params[2][current_cache], 
		  zoom_params[3][current_cache], 1 );
  */
}




int main( int argc, char *argv[] ) {

#ifdef USE_C_PLUS_PLUS
  curState = new state();
#else
  curState = (struct state*)malloc( sizeof(struct state ) );
#endif
  
  /*** some default values ***/
  curState->width = 50;
  curState->height = 50;
  curState->MAXITS = 100;
  curState->depth = 1;
  curState->type =  0;
  curState->subA.a = -0.3456;
  curState->subA.b = -0.5768;
  
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
  glutCreateWindow(version);
  glutMouseFunc(MouseInputHandler);
  glutMotionFunc(MouseMoveHandler);
  glutDisplayFunc(display);
  glutReshapeFunc(reshape);
  glutKeyboardFunc(keyboard);
  init();

  /*** filename cmd line parameter ***/
  if ( argv[1] != NULL ) {
    colorcount = 0;
    printf( "loading %s.\n", argv[1] );
    ReadFile( argv[1] ); 	
  }

 
  /*** set up initial state(but not quite...) ***/
  current_zoom_params[0] = zoom_params[0][current_cache];
  current_zoom_params[1] = zoom_params[1][current_cache];
  current_zoom_params[2] = zoom_params[2][current_cache];
  current_zoom_params[3] = zoom_params[3][current_cache];
  
  create_fractal( zoom_params[0][current_cache], 
		  zoom_params[1][current_cache], 
		  zoom_params[2][current_cache], 
		  zoom_params[3][current_cache], 1 );

  glutMainLoop();
  return 0;             
}
